package org.example.models.trading;

import simudyne.core.abm.Action;
import simudyne.core.abm.Agent;


public abstract class BaseTrader extends Agent<Globals> {
    /**
     * The base trader class incorporates general methods that all traders will use (e.g. submitOrder) as well as
     * abstract methods that will be implemented specifically for each trader class (e.g. get demand)
     */
    public static Action<BaseTrader> updateMarketInformation() {
        return Action.create(
                BaseTrader.class, baseTrader -> {
                    baseTrader.getMessagesOfType(Messages.MarketChange.class)
                            .forEach(baseTrader::handleMarketInformation);
                });
    }


    public abstract void handleMarketInformation(Messages.MarketChange msg);


    public abstract Side getSide();


    public abstract double getDemand();


    public static Action<BaseTrader> submitOrder() {
        return Action.create(
                BaseTrader.class, baseTrader -> {
                    Side side = baseTrader.getSide();
                    double vol = baseTrader.getDemand();
                    assert (vol >= 0);
                    if (side == Side.BUY)
                        baseTrader.buy(vol);
                    else if (side == Side.SELL)
                        baseTrader.sell(vol);
                });
    }


    protected void buy(double vol) {
        getLinks(Links.TradeLink.class).send(Messages.BuyOrderPlaced.class, vol);
    }


    protected void sell(double vol) {
        getLinks(Links.TradeLink.class).send(Messages.SellOrderPlaced.class, vol);
    }
}
