package org.example.models.trading;

import simudyne.core.abm.Action;
import simudyne.core.abm.Agent;
import simudyne.core.annotations.Variable;
import simudyne.core.functions.SerializableConsumer;
import simudyne.core.graph.Message;

public class Market extends Agent<Globals> {

    @Variable
    public double price;

    @Variable
    public double fundamentalValue;

    @Variable
    public double momentum;

    @Variable
    public double excessDemand;

    @Variable
    public double CumulativeAbsoluteDemand;

    @Variable
    public double CummulativeExcessDemand;

    private double lastMarketPrice;


    public static Action<Market> calcPriceImpact() {
        return action(
                market -> {
                    double demand = market.getMessagesOfType(Messages.BuyOrderPlaced.class).stream().mapToDouble(Message.Double::getBody).sum();
                    double supply = market.getMessagesOfType(Messages.SellOrderPlaced.class).stream().mapToDouble(Message.Double::getBody).sum();

                    System.out.println("demand: " + demand + ", supply: " + supply);
                    market.excessDemand = demand - supply;
                    market.CumulativeAbsoluteDemand += demand + supply;
                    market.CummulativeExcessDemand += market.excessDemand;
                    double marketImpact = market.excessDemand * market.getGlobals().lambda;
                    market.price = Math.max(0, market.price + marketImpact);
                    market.updateMomentum(market.price);

                    market.getLinks(Links.TradeLink.class).send(Messages.MarketChange.class, (msg,link)->{
                        msg.price = market.price;
                        msg.momentum = market.momentum;
                    });

                    market.fundamentalValue = market.getGlobals().v;
                });
    }

    public static Action<Market> publish() {
        return action(
                market -> {
                    market.getDoubleAccumulator("market_price").add(market.price);
                });
    }



    public void updateMomentum(double prc) {
        if(lastMarketPrice > 0)
            momentum = getGlobals().alpha * (prc - lastMarketPrice) + (1.0 - getGlobals().alpha) * momentum;

        lastMarketPrice = prc;
    }


    private static Action<Market> action(SerializableConsumer<Market> consumer) {
        return Action.create(Market.class, consumer);
    }
}
