package org.example.models.trading;

import simudyne.core.abm.AgentBasedModel;
import simudyne.core.abm.GlobalState;
import simudyne.core.abm.Group;
import simudyne.core.annotations.Constant;
import simudyne.core.annotations.Input;
import simudyne.core.annotations.ModelSettings;
import simudyne.core.annotations.Variable;

import java.util.Random;

@ModelSettings(macroStep = 100)
public class TradingModel extends AgentBasedModel<Globals> {

  @Override
  public void init()
  {
    createDoubleAccumulator("market_price");
    registerAgentTypes(Market.class, NoiseTrader.class, FundamentalTrader.class, MomentumTrader.class);
    registerLinkTypes(Links.TradeLink.class);
  }


  @Override
  public void setup() {
    Group<NoiseTrader> noiseGroup = generateGroup(NoiseTrader.class, getGlobals().nbNoiseTraders, trader -> {
      trader.sigma_n = getGlobals().sigma_n;
      trader.probability_order = 1 / (double) getGlobals().nbNoiseTraders;

    });

    Group<FundamentalTrader> ftGroup = generateGroup(FundamentalTrader.class, getGlobals().nbFundamentalTraders, trader -> {
      trader.kappa = getGlobals().kappa / (double) getGlobals().nbFundamentalTraders;
      trader.kappa_3 = getGlobals().kappa_3 / (double) getGlobals().nbFundamentalTraders;
    });

    Group<MomentumTrader> mtGroup = generateGroup(MomentumTrader.class, getGlobals().nbMomentumTraders, trader -> {
      trader.beta = getGlobals().beta / (double) getGlobals().nbMomentumTraders;
      trader.gamma = getGlobals().gamma;
    });

    Group<Market> marketGroup = generateGroup(Market.class, 1, market -> {
      market.price = getGlobals().v0 - getGlobals().priceDistortion;
      market.fundamentalValue=getGlobals().v0;
      market.momentum = getGlobals().m0;
    });

    noiseGroup.fullyConnected(marketGroup, Links.TradeLink.class);
    marketGroup.fullyConnected(noiseGroup, Links.TradeLink.class);

    ftGroup.fullyConnected(marketGroup, Links.TradeLink.class);
    marketGroup.fullyConnected(ftGroup, Links.TradeLink.class);

    mtGroup.fullyConnected(marketGroup, Links.TradeLink.class);
    marketGroup.fullyConnected(mtGroup, Links.TradeLink.class);

    getGlobals().init(getContext().getPrng().generator.nextLong());
    super.setup();
  }


  @Override
  public void step() {
    super.step();

    run(BaseTrader.submitOrder(), Market.calcPriceImpact(), BaseTrader.updateMarketInformation());
    run(Market.publish());

    getGlobals().step();
  }
}
